# coding: utf-8

from ...characterType import numeric, alphaNumeric
from ...row import Row


class Trailer:

    def __init__(self):
        self.content = "00000005         000003000000000000000000000000000000000000                                                                                                                                                                                     "

    def setAmountInCents(self, amount):
        structs = [
            (23, 41, 18, numeric, amount),                     # Soma dos valores dos lotes
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0, 3, 3, numeric, bank.bankId),                   # Código do banco debitado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (3, 7, 4, numeric, index)                          # Indica index do lote
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)


from ...libs.dac import DAC
from .segmentP import SegmentP
from .segmentQ import SegmentQ


class SlipV30:

    def __init__(self):
        self.segmentP=SegmentP()
        self.segmentQ=SegmentQ()

    def toString(self):
        return "%s\r\n%s" % (self.segmentP.content, self.segmentQ.content)

    def amountInCents(self):
        return self.segmentP.amountInCents()

    def setPositionInLot(self, index):
        self.segmentP.setPositionInLot(index)
        self.segmentQ.setPositionInLot(index+1)

    def setSender(self, user):
        self.segmentP.setSenderBank(user.bank)
        self.segmentQ.setSenderBank(user.bank)

    def setPayer(self, user):
        self.segmentQ.setPayer(user)
        self.segmentQ.setPayerAddress(user.address)

    def setAmountInCents(self, value):
        self.segmentP.setAmountInCents(value)

    def setExpirationDate(self, datetime):
        self.segmentP.setExpirationDate(datetime.strftime("%d%m%Y"))

    def setIssueDate(self, datetime):
        self.segmentP.setIssueDate(datetime.strftime("%d%m%Y"))

    def setBankIdentifier(self, identifier, branch, accountNumber, wallet):
        dac = DAC.calculate(
            branch="{:0>4}".format(branch)[:4],
            accountNumber=accountNumber,
            wallet="{:0>3}".format(wallet)[:3],
            bankNumber="{:0>8}".format(identifier)[:8]
        )
        self.segmentP.setBankIdentifier(identifier, dac)

    def setIdentifier(self, identifier):
        self.segmentP.setIdentifier(identifier)


import os.path


class FileUtils:

    @classmethod
    def create(cls, name, path):
        projectFolder = os.path.abspath(os.path.join(__file__, "../.."))
        fileAddress = projectFolder + path + name
        return open(fileAddress, 'w+')


from re import match
from ...characterType import alphaNumeric, numeric


def validateFormatter(func):
   def wrapper(self, string, charactersType, numberOfCharacters, defaultCharacter):
       if type(numberOfCharacters) != int:
           raise ValueError("numberOfCharacters must be Integer")

       if charactersType not in [alphaNumeric, numeric]:
           raise ValueError("charactersType must be alphaNumeric or numeric")

       if not isinstance(string, str):
           raise ValueError("(%s,%s) is not an allowed value. It must be string" % (str(string), str(type(string))))

       regex = {
           alphaNumeric: r'^[A-Za-z0-9\s\-]+$',
           numeric:      r'^[0-9]+$'
       }[charactersType]
       if string and not match(regex, string):
           raise ValueError("You add %s that is a non %s value" % (string, charactersType))
       return func(self, string, charactersType, numberOfCharacters, defaultCharacter)
   return wrapper


from febraban.cnab240.row import Row
from ...characterType import numeric, alphaNumeric


class Header:

    def __init__(self):
        self.content = "00000000         000000000000000                    00000 000000000000 0                                                                      00000000000000000000000000000                                                      000            "
        self.defaultValues()

    def defaultValues(self):
        structs = [
            (142, 143, 1, numeric, "1"),    # 1 = Remessa, 2 = Retorno
            (163, 166, 3, numeric, "040"),  # Layout do arquivo
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setGeneratedFileDate(self, date, time):
        structs = [
            (143, 151, 8, numeric, date),   # Dia que o arquivo foi gerado
            (151, 157, 6, numeric, time),   # Horario que o arquivo foi gerado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSender(self, user):
        structs = [
            (17,  18,  1,      numeric, "1" if len(user.identifier) == 11 else "2"),
            (18,  32, 14,      numeric, user.identifier),
            (72, 102, 30, alphaNumeric, user.name)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (  0,   3,  3,      numeric, bank.bankId),
            ( 52,  57,  5,      numeric, bank.branchCode),
            ( 58,  70, 12,      numeric, bank.accountNumber),
            ( 71,  72,  1,      numeric, bank.accountVerifier),
            (102, 132, 30, alphaNumeric, bank.bankName)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from ...libs.fileUtils import FileUtils
from .header import Header
from .trailer import Trailer


class FileV83:

    def __init__(self):
        self.header = Header()
        self.lots = []
        self.trailer = Trailer()

    def add(self, lot):
        lot.setPositionInLot(index=len(self.lots)+1)
        self.lots.append(lot.toString())

    def toString(self):
        self.header.setGeneratedFileDate()
        self.trailer.setNumberOfLotsAndRegisters(num=len(self.lots))
        lotsToString = "\r\n".join(self.lots)
        return "%s\r\n%s\r\n%s\r\n" % (self.header.content, lotsToString, self.trailer.content)

    def setSender(self, user):
        self.header.setSender(user)
        self.header.setSenderBank(user.bank)
        self.trailer.setSenderBank(user.bank)

    def output(self, fileName, path="/../", content=None):
        file = FileUtils.create(name=fileName, path=path)
        file.write(self.toString() if not content else content)
        file.close()




class DAC:

    @classmethod
    def calculate(cls, branch, accountNumber, wallet, bankNumber):
        x1 = branch + accountNumber + wallet + bankNumber
        x2 = "12"*(len(x1)/2) + ("1" if len(x1)%2 == 1 else "")
        out = [int(a) * int(b) for (a,b) in zip(x1, x2)]
        s = sum(map(lambda x: x/10 + x%10, out))
        return (10 - s%10)%10


from .occurrences import occurrences


class PaymentResponseStatus:

    success = "success"
    failed = "failed"
    scheduled = "scheduled"
    unknown = "unknown"


class PaymentResponse:

    def __init__(self, identifier=None, occurrences=None, content=None, authentication=None):
        self.identifier = identifier
        self.occurrences = occurrences
        self.content = content or []
        self.authentication = authentication

    def occurrencesText(self):
        return [occurrences[occurrenceId] for occurrenceId in self.occurrences]

    def occurrencesTextAtIndex(self, index):
        occurrenceId = self.occurrences[index]
        return occurrences[occurrenceId]

    def status(self):
        if "00" in self.occurrences:
            return PaymentResponseStatus.success
        if "BD" in self.occurrences:
            return PaymentResponseStatus.scheduled
        if [code in self.occurrences for code in ["RJ", "DV"]].count(True) > 0:
            return PaymentResponseStatus.failed
        return PaymentResponseStatus.unknown

    def contentText(self):
        return "".join(self.content)


class PaymentParser:

    @classmethod
    def parseFile(cls, file):
        lines = file.readlines()
        return cls._parseLines(lines)

    @classmethod
    def parseText(cls, text):
        lines = text.splitlines()[:-1]
        return cls._parseLines(lines)

    @classmethod
    def _parseLines(cls, lines):
        result = []
        for line in lines:
            if line[7] in ["0","9"]:
                continue
            elif line[7] == "1":
                currentResponse = PaymentResponse(content=[line])
            elif line[7] == "3" and line[13] == "A":
                currentResponse.content.append(line)
                currentResponse.identifier = cls._getIdentifier(line)
                currentResponse.occurrences = cls._getOccurrences(line)
            elif line[7] == "3" and line[13] == "Z":
                currentResponse.content.append(line)
                currentResponse.authentication = cls._getAuthentication(line)
            elif line[7] == "5":
                currentResponse.content.append(line)
                result.append(currentResponse)
                currentResponse = PaymentResponse()
        return result

    @classmethod
    def _getOccurrences(cls, line):
        occurrencesString = line[230:240].strip()
        return cls._splitString(occurrencesString)

    @classmethod
    def _splitString(cls, string):
        return [string[i:i+2] for i in range(0, len(string), 2)]

    @classmethod
    def _getIdentifier(self, line):
        return line[73:93].strip()

    @classmethod
    def _getAuthentication(cls, line):
        return line[14:78].strip()


from .header import Header
from .segmentA import SegmentA
from .trailer import Trailer


class PaymentV83:

    def __init__(self):
        self.header = Header()
        self.segmentA = SegmentA()
        self.trailer = Trailer()

    def toString(self):
        return self.header.content + "\r\n" + self.segmentA.content + "\r\n" + self.trailer.content

    def setSender(self, user):
        self.header.setSender(user)
        self.header.setSenderBank(user.bank)
        self.header.setSenderAddress(user.address)
        self.segmentA.setSenderBank(user.bank)
        self.trailer.setSenderBank(user.bank)

    def setReceiver(self, user):
        self.segmentA.setReceiver(user)
        self.segmentA.setReceiverBank(user.bank)

    def setAmountInCents(self, value):
        self.segmentA.setAmountInCents(value)
        self.trailer.setAmountInCents(value)

    def setPositionInLot(self, index):
        self.header.setPositionInLot(index)
        self.segmentA.setPositionInLot(index)
        self.trailer.setPositionInLot(index)

    def setScheduleDate(self, date):
        self.segmentA.setScheduleDate(date)

    def setInfo(self, kind="98", method="41", reason="10"):
        """
        This method set config information in the payment

        Args:
            kind:   String - Kind of payment - 98 Diversos, read: NOTES 4
            method: String - Payment method  - 41 TED Outro titular, 43 TED Mesmo titular, 01 ITAU account. read: NOTES 5
            reason: String - Payment reason  - 10 Credito em Conta Corrente, read: NOTES 26
        """
        self.header.setInfo(kind, method)
        self.segmentA.setInfo(reason)

    def setIdentifier(self, identifier):
        self.segmentA.setIdentifier(identifier)


# coding: utf-8

from ...row import Row
from ...characterType import numeric


class Trailer:

    def __init__(self):
        self.content = "00099999         000001000005                                                                                                                                                                                                                   "

    def setNumberOfLotsAndRegisters(self, num):
        structs = [
            (17, 23, 6, numeric, num),
            (23, 29, 6, numeric, 2+3*num),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0,   3,  3, numeric, bank.bankId),              # Código do banco debitado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)


# coding: utf-8

from ...row import Row
from ...characterType import numeric, alphaNumeric


class Header:

    def __init__(self):
        self.content = "00000001C0000040 000000000000000                    00000 000000000000 0                                                                                                    00000                                   00000000                    "

    def setSender(self, user):
        structs = [
            (17,  18,  1,      numeric, "1" if len(user.identifier) == 11 else "2"),
            (18,  32, 14,      numeric, user.identifier),
            (72, 102, 30, alphaNumeric, user.name)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            ( 0,   3,  3, numeric, bank.bankId),
            (52,  57,  5, numeric, bank.branchCode),
            (58,  70, 12, numeric, bank.accountNumber),
            (71,  72,  1, numeric, bank.accountVerifier),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderAddress(self, address):
        structs = [
            (142, 172, 30, alphaNumeric, address.streetName),
            (172, 177,  5,      numeric, address.number),
            (192, 212, 20, alphaNumeric, address.city),
            (212, 220,  8,      numeric, address.zipcode),
            (220, 222,  2, alphaNumeric, address.state),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (3, 7, 4, numeric, index)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setInfo(self, kind, method):
        structs = [
            ( 9, 11, 2, numeric, kind),
            (11, 13, 2, numeric, method)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)




class User:

    def __init__(self, name, identifier, bank=None, address=None):
        self.name = name
        self.identifier = identifier
        self.bank = bank
        self.address = address


class UserBank:

    def __init__(self, bankId, branchCode, accountNumber, accountVerifier, bankName=""):
        self.bankId = bankId
        self.bankName = bankName
        self.accountNumber = accountNumber
        self.branchCode = branchCode
        self.accountVerifier = accountVerifier


class UserAddress:

    def __init__(self, streetName, number, city, state, zipcode, district="", complement=""):
        self.streetName = streetName
        self.number = number
        self.complement = complement
        self.district = district
        self.city = city
        self.state = state
        self.zipcode = zipcode


# coding: utf-8

from ...characterType import numeric, alphaNumeric
from ...row import Row


class SegmentP:

    def __init__(self):
        self.content = "0000000000000  0000000 000000000000 0000000000000        00000               0000000000000000000000000000000A00000000000000000000000000000000000000000000000000000000000000000000000000000000000000                         0000000000000000000 "
        self.defaultValues()

    def amountInCents(self):
        return int(self.content[85:100])

    def defaultValues(self):
        structs = [
            (  3,   7, 4,      numeric, "1"),
            (  7,   8, 1,      numeric, "3"),
            ( 13,  14, 1, alphaNumeric, "P"),
            ( 15,  17, 2,      numeric, "01"),
            ( 37,  40, 3,      numeric, "109"),               # Numero da Carteira de Cobranca
            (106, 108, 2,      numeric, "99"),                # Especie do Titulo: 99 = Real
            (108, 109, 1, alphaNumeric, "A"),                 # Aceite: A = Sim, N = Nao
            (223, 224, 1,      numeric, "1"),                 # 0 - Não baixe boletos, 1 - Baixe boletos após x dias do vencimento
            (224, 226, 2,      numeric, "59"),                # Quantidade de dias após o vencimento para baixa automática
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (  0,   3,  3,      numeric, bank.bankId),
            ( 18,  22,  4,      numeric, bank.branchCode),
            ( 23,  35, 12,      numeric, bank.accountNumber),
            ( 36,  37,  1,      numeric, bank.accountVerifier)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (8, 13, 5, numeric, index)                        # Indica index do lote
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setBankIdentifier(self, identifier, dac):
        structs = [
            (40, 48, 8, numeric, identifier),                # Numero dado pelo Banco
            (48, 49, 1, numeric, dac)                        # Verificador do numero dado pelo Banco
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setAmountInCents(self, amount):
        structs = [
            (85, 100, 15, numeric, amount),                  # Valor do Boleto
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setExpirationDate(self, date):
        structs = [
            (77, 85, 8, numeric, date),                      # Data de vencimento do boleto
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setIssueDate(self, date):
        structs = [
            (109, 117, 8, numeric, date),                    # Data de emissao do boleto
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setIdentifier(self, identifier):
        structs = [
            (195, 220, 25, alphaNumeric, identifier)           # Id da empresa na transação
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)


from .occurrences import occurrences


class SlipResponseStatus:

    registered = "registered"
    paid = "paid"
    overdue = "overdue"
    failed = "failed"
    unknown = "unknown"


class SlipResponse:

    def __init__(self, identifier=None, occurrences=None, content=None, amountInCents=None):
        self.identifier = identifier
        self.occurrences = occurrences
        self.amountInCents = amountInCents
        self.content = content or []

    def occurrencesText(self):
        return [occurrences[occurrenceId] for occurrenceId in self.occurrences]

    def occurrencesTextAtIndex(self, index):
        occurrenceId = self.occurrences[index]
        return occurrences[occurrenceId]

    def status(self):
        if "02" in self.occurrences:
            return SlipResponseStatus.registered
        if "03" in self.occurrences:
            return SlipResponseStatus.failed
        if "06" in self.occurrences:
            return SlipResponseStatus.paid
        return SlipResponseStatus.unknown

    def contentText(self):
        return "".join(self.content)


class SlipParser:

    @classmethod
    def parseFile(cls, file):
        lines = file.readlines()
        return cls._parseLines(lines)

    @classmethod
    def parseText(cls, text):
        lines = text.splitlines()[:-1]
        return cls._parseLines(lines)

    @classmethod
    def _parseLines(cls, lines):
        result = []
        for line in lines:
            if line[7] == "1":
                currentResponse = SlipResponse()
            elif line[7] == "3":
                currentResponse.content.append(line)
            if line[13] == "T":
                currentResponse.amountInCents = int(line[81:96])
                currentResponse.occurrences = [line[15:17]]
                currentResponse.identifier = line[105:130].strip()
            elif line[13] == "U":
                result.append(currentResponse)
                currentResponse = SlipResponse()
            elif line[13] == "P":
                currentResponse.amountInCents = int(line[85:100])
                currentResponse.occurrences = [line[15:17]]
                currentResponse.identifier = line[62:72].strip()
            elif line[13] == "Q":
                result.append(currentResponse)
                currentResponse = SlipResponse()
        return result


# coding: utf-8

from ...row import Row
from ...characterType import numeric, alphaNumeric


class Header:

    def __init__(self):
        self.content = "00000000      081000000000000000                    00000 000000000000 0                                                                      10312201719400900000000000000                                                                     "

    def setGeneratedFileDate(self):
        import datetime
        now = datetime.datetime.now()
        structs = [
            (143, 151, 8, numeric, now.strftime("%d%m%Y")),   # Dia que o arquivo foi gerado
            (151, 157, 6, numeric, now.strftime("%H%M%S")),   # Horario que o arquivo foi gerado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSender(self, user):
        structs = [
            (17,  18,  1,      numeric, "1" if len(user.identifier) == 11 else "2"),
            (18,  32, 14,      numeric, user.identifier),
            (72, 102, 30, alphaNumeric, user.name)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            ( 0,   3,  3, numeric, bank.bankId),
            (52,  57,  5, numeric, bank.branchCode),
            (58,  70, 12, numeric, bank.accountNumber),
            (71,  72,  1, numeric, bank.accountVerifier),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from datetime import datetime
from ...libs.fileUtils import FileUtils
from .header import Header
from .headerLot import HeaderLot
from .trailer import Trailer
from .trailerLot import TrailerLot


class FileV30:

    def __init__(self):
        self.header = Header()
        self.headerLot = HeaderLot()
        self.registers = []
        self.trailerLot = TrailerLot()
        self.trailer = Trailer()
        self.issueDate = datetime.now()
        self.amount = 0
        self.index = 1

    def add(self, register):
        register.setIssueDate(datetime=self.issueDate)
        register.setPositionInLot(index=self.index)
        self.registers.append(register.toString())
        self.amount += register.amountInCents()
        self.index += 2

    def toString(self):
        self.trailer.setNumberOfLotsAndRegisters(num=len(self.registers))
        self.trailerLot.setNumberOfLotsAndRegisters(num=len(self.registers))
        self.trailerLot.setSumOfValues(sum=self.amount)
        lotsToString = "\r\n".join(self.registers)
        return "%s\r\n%s\r\n%s\r\n%s\r\n%s\r\n" % (
            self.header.content,
            self.headerLot.content,
            lotsToString,
            self.trailerLot.content,
            self.trailer.content
        )

    def setSender(self, sender):
        self.header.setSender(sender)
        self.header.setSenderBank(sender.bank)
        self.headerLot.setSender(sender)
        self.headerLot.setSenderBank(sender.bank)
        self.trailer.setSenderBank(sender.bank)
        self.trailerLot.setSenderBank(sender.bank)

    def setIssueDate(self, datetime):
        issueTime = datetime.strftime("%H%M%S")
        issueDate = datetime.strftime("%d%m%Y")
        self.header.setGeneratedFileDate(date=issueDate, time=issueTime)
        self.headerLot.setGeneratedFileDate(date=issueDate)

    def output(self, fileName, path="/../", content=None):
        file = FileUtils.create(name=fileName, path=path)
        file.write(self.toString() if not content else content)
        file.close()


import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), "README.srt")) as readme:
    README = readme.read()

setup(
    name="febraban",
    packages=find_packages(),
    include_package_data=True,
    description="A library to generate files in FEBRABAN formats",
    long_description=README,
    license="MIT License",
    url='https://github.com/starkbank/febraban.git',
    author="Stark Bank",
    author_email="rafael@starkbank.com",
    keywords=["febraban", "cnab", "cnab 240", "cnab240", "febraban240", "transfer", "billing", "bank"],
    version = "0.1.20"
)


# coding: utf-8

occurrences = {
    "02": "ENTRADA CONFIRMADA (COM POSSIBILIDADE DE MENSAGEM – NOTA 23 – TABELA 8)",
    "03": "ENTRADA REJEITADA (NOTA 23 - TABELA 1)",
    "04": "ALTERAÇÃO DE DADOS – NOVA ENTRADA OU ALTERAÇÃO/EXCLUSÃOADOS ACATADA",
    "05": "ALTERAÇÃO DE DADOS – BAIXA",
    "06": "LIQUIDAÇÃO NORMAL",
    "08": "LIQUIDAÇÃO EM CARTÓRIO",
    "09": "BAIXA SIMPLES",
    "10": "BAIXA POR TER SIDO LIQUIDADO",
    "11": "EM SER (SÓ NO RETORNO MENSAL)",
    "12": "ABATIMENTO CONCEDIDO",
    "13": "ABATIMENTO CANCELADO",
    "14": "VENCIMENTO ALTERADO",
    "15": "BAIXAS REJEITADAS (NOTA 23 - TABELA 4)",
    "16": "INSTRUÇÕES REJEITADAS (NOTA 23 – TABELA 3)",
    "17": "ALTERAÇÃO/EXCLUSÃO DE DADOS REJEITADA (NOTA 23 - TABELA 2)",
    "18": "COBRANÇA CONTRATUAL – INSTRUÇÕES/ALTERAÇÕES REJEITADAS/PENDENTES (NOTA 23 - TABELA 5)",
    "19": "CONFIRMAÇÃO RECEBIMENTO DE INSTRUÇÃO DE PROTESTO",
    "20": "CONFIRMAÇÃO RECEBIMENTO DE INSTRUÇÃO DE SUSTAÇÃO DE PROTESTO /TARIFA",
    "21": "CONFIRMAÇÃO RECEBIMENTO DE INSTRUÇÃO DE NÃO PROTESTAR",
    "23": "PROTESTO ENVIADO A CARTÓRIO/TARIFA",
    "24": "INSTRUÇÃO DE PROTESTO SUSTADA (NOTA 23 - TABELA 7)",
    "25": "ALEGAÇÕES DO PAGADOR (NOTA 23 - TABELA 6)",
    "26": "TARIFA DE AVISO DE COBRANÇA",
    "27": "TARIFA DE EXTRATO POSIÇÃO (B40X)",
    "28": "TARIFA DE RELAÇÃO DAS LIQUIDAÇÕES",
    "29": "TARIFA DE MANUTENÇÃO DE TÍTULOS VENCIDOS",
    "30": "DÉBITO MENSAL DE TARIFAS (PARA ENTRADAS E BAIXAS)",
    "32": "BAIXA POR TER SIDO PROTESTADO",
    "33": "CUSTAS DE PROTESTO",
    "34": "CUSTAS DE SUSTAÇÃO",
    "35": "CUSTAS DE CARTÓRIO DISTRIBUIDOR",
    "36": "CUSTAS DE EDITAL",
    "37": "TARIFA DE EMISSÃO DE BOLETO/TARIFA DE ENVIO DE DUPLICATA",
    "38": "TARIFA DE INSTRUÇÃO",
    "39": "TARIFA DE OCORRÊNCIAS",
    "40": "TARIFA MENSAL DE EMISSÃO DE BOLETO/TARIFA MENSAL DE ENVIO DE DUPLICATA",
    "41": "DÉBITO MENSAL DE TARIFAS – EXTRATO DE POSIÇÃO (B4EP/B4OX)",
    "42": "DÉBITO MENSAL DE TARIFAS – OUTRAS INSTRUÇÕES",
    "43": "DÉBITO MENSAL DE TARIFAS – MANUTENÇÃO DE TÍTULOS VENCIDOS",
    "44": "DÉBITO MENSAL DE TARIFAS – OUTRAS OCORRÊNCIAS",
    "45": "DÉBITO MENSAL DE TARIFAS – PROTESTO",
    "46": "DÉBITO MENSAL DE TARIFAS – SUSTAÇÃO DE PROTESTO",
    "47": "BAIXA COM TRANSFERÊNCIA PARA DESCONTO",
    "48": "CUSTAS DE SUSTAÇÃO JUDICIAL",
    "51": "TARIFA MENSAL REFERENTE A ENTRADAS BANCOS CORRESPONDENTES NA CARTEIRA",
    "52": "TARIFA MENSAL BAIXAS NA CARTEIRA",
    "53": "TARIFA MENSAL BAIXAS EM BANCOS CORRESPONDENTES NA CARTEIRA",
    "54": "TARIFA MENSAL DE LIQUIDAÇÕES NA CARTEIRA",
    "55": "TARIFA MENSAL DE LIQUIDAÇÕES EM BANCOS CORRESPONDENTES NA CARTEIRA",
    "56": "CUSTAS DE IRREGULARIDADE",
    "57": "INSTRUÇÃO CANCELADA (NOTA 23 – TABELA 8)",
    "60": "ENTRADA REJEITADA CARNÊ (NOTA 20 – TABELA 1)",
    "61": "TARIFA EMISSÃO AVISO DE MOVIMENTAÇÃO DE TÍTULOS (2154)",
    "62": "DÉBITO MENSAL DE TARIFA – AVISO DE MOVIMENTAÇÃO DE TÍTULOS (2154)",
    "63": "TÍTULO SUSTADO JUDICIALMENTE",
    "74": "INSTRUÇÃO DE NEGATIVAÇÃO EXPRESSA REJEITADA (NOTA 25 – TABELA 3)",
    "75": "CONFIRMA O RECEBIMENTO DE INSTRUÇÃO DE ENTRADA EM NEGATIVAÇÃO EXPRESSA",
    "77": "CONFIRMA O RECEBIMENTO DE INSTRUÇÃO DE EXCLUSÃO DE ENTRADA EM NEGATIVAÇÃO EXPRESSA",
    "78": "CONFIRMA O RECEBIMENTO DE INSTRUÇÃO DE CANCELAMENTO DA NEGATIVAÇÃO EXPRESSA",
    "79": "NEGATIVAÇÃO EXPRESSA INFORMACIONAL (NOTA 25 – TABELA 12)",
    "80": "CONFIRMAÇÃO DE ENTRADA EM NEGATIVAÇÃO EXPRESSA – TARIFA",
    "82": "CONFIRMAÇÃO O CANCELAMENTO DE NEGATIVAÇÃO EXPRESSA - TARIFA",
    "83": "CONFIRMAÇÃO DA EXCLUSÃO/CANCELAMENTO DA NEGATIVAÇÃO EXPRESSA POR LIQUIDAÇÃO - TARIFA",
    "85": "TARIFA POR BOLETO (ATÉ 03 ENVIOS) COBRANÇA ATIVA ELETRÔNICA",
    "86": "TARIFA EMAIL COBRANÇA ATIVA ELETRÔNICA",
    "87": "TARIFA SMS COBRANÇA ATIVA ELETRÔNICA",
    "88": "TARIFA MENSAL POR BOLETO (ATÉ 03 ENVIOS) COBRANÇA ATIVA ELETRÔNICA",
    "89": "TARIFA MENSAL EMAIL COBRANÇA ATIVA ELETRÔNICA",
    "90": "TARIFA MENSAL SMS COBRANÇA ATIVA ELETRÔNICA",
    "91": "TARIFA MENSAL DE EXCLUSÃO DE ENTRADA EM NEGATIVAÇÃO EXPRESSA",
    "92": "TARIFA MENSAL DE CANCELAMENTO DE NEGATIVAÇÃO EXPRESSA",
    "93": "TARIFA MENSAL DE EXCLUSÃO/CANCELAMENTO DE NEGATIVAÇÃO EXPRESSA POR LIQUIDAÇÃO",
    "94": "CONFIRMA RECEBIMENTO DE INSTRUÇÃO DE NÃO NEGATIVAR",
}